#!/usr/bin/env node

'use strict';

var AppStore = require('../appstore.js'),
    assert = require('assert'),
    Cloudron = require('../cloudron.js'),
    path = require('path'),
    readlineSync = require('readline-sync'),
    request = require('superagent-sync'),
    semver = require('semver'),
    util = require('util');

require('colors');

process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0';

function verifyResponse(res, errorMessage) {
    if (res.statusCode < 200 || res.statusCode > 299) {
        console.log('Response error statusCode:%s error:%s body:%s', res.statusCode, res.error, res.body);
        console.log(errorMessage.red);
        process.exit(1);
    }
}

describe('End to end testing', function () {
    this.timeout(0);

    var appStore = new AppStore('https://api.staging.cloudron.io');

    var owner = {
        username: 'test',
        password: 'test1234',
        email: 'test@cloudron.io'
    };

    var res, latestVersion, cloudron, appId, box;

    it('can query versions', function () {
        res = request.get('https://s3.amazonaws.com/staging-cloudron-releases/versions.json').end();
        verifyResponse(res);
        var boxVersions = Object.keys(res.body).sort(semver.rcompare);
        latestVersion = boxVersions[0];
    });

    it('can login to the store', function () {
        var accessToken = appStore.getAccessToken(owner);
        appStore.setCredentials(owner.password, accessToken);
    });

    it('can create a cloudron', function () {
        box = appStore.createCloudron({
            name: 'testbox-' + path.basename(__filename, '-test.js') + '.smartserver.io',
            zoneName: 'smartserver.io',
            region: 'sfo1',
            size: '512mb',
            version: latestVersion
        });
        cloudron = new Cloudron(box);
    });

    it('can activate the box', function () {
        cloudron.activate(owner);
    });

    it('can login to the box', function () {
        var token = cloudron.getOauthToken(owner);
        cloudron.setCredentials(owner.password, token);
    });

    var location = 'haste' + (Math.random() * 10000).toFixed();
    it('can install app', function () {
        var manifest = appStore.getManifest('com.hastebin.cloudronapp', '0.1.0');
        appId = cloudron.installApp(location, manifest);
    });

    it('can configure app', function () {
        cloudron.configureApp(appId, location + 'x');
    });

    it('can uninstall app', function () {
        cloudron.uninstallApp(appId);
    });

    it('can delete the cloudron', function () {
        appStore.deleteCloudron(box);
    });

});

